@@ -17,6 +17,8 @@ import (
 	configinformers "github.com/openshift/client-go/config/informers/externalversions"
 	configlisterv1 "github.com/openshift/client-go/config/listers/config/v1"
 	"github.com/openshift/cluster-kube-apiserver-operator/pkg/operator/operatorclient"
+	"github.com/openshift/library-go/pkg/controller/factory"
+
 	"github.com/openshift/library-go/pkg/operator/certrotation"
 	"github.com/openshift/library-go/pkg/operator/events"
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
@@ -26,7 +28,7 @@ import (
 const defaultRotationDay = 24 * time.Hour
 
 type CertRotationController struct {
-	certRotators []*certrotation.CertRotationController
+	certRotators []factory.Controller
 
 	networkLister        configlisterv1.NetworkLister
 	infrastructureLister configlisterv1.InfrastructureLister
@@ -40,6 +42,8 @@ type CertRotationController struct {
 	internalLoadBalancer               *DynamicServingRotation
 	internalLoadBalancerHostnamesQueue workqueue.RateLimitingInterface
 
+	recorder events.Recorder
+
 	cachesToSync []cache.InformerSynced
 }
 
@@ -103,6 +107,7 @@ func newCertRotationController(
 		internalLoadBalancerHostnamesQueue: workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "InternalLoadBalancerHostnames"),
 		internalLoadBalancer:               &DynamicServingRotation{hostnamesChanged: make(chan struct{}, 10)},
 
+		recorder: eventRecorder,
 		cachesToSync: []cache.InformerSynced{
 			configInformer.Config().V1().Networks().Informer().HasSynced,
 			configInformer.Config().V1().Infrastructures().Informer().HasSynced,
@@ -119,7 +124,7 @@ func newCertRotationController(
 		klog.Warningf("Certificate rotation base set to %q", rotationDay)
 	}
 
-	certRotator, err := certrotation.NewCertRotationController(
+	certRotator := certrotation.NewCertRotationController(
 		"AggregatorProxyClientCert",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -155,13 +160,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"KubeAPIServerToKubeletClientCert",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -197,13 +200,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"LocalhostServing",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -239,13 +240,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"ServiceNetworkServing",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -282,13 +281,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"ExternalLoadBalancerServing",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -325,13 +322,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"InternalLoadBalancerServing",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -368,13 +363,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"LocalhostRecoveryServing",
 		certrotation.SigningRotation{
 			Namespace:     operatorclient.OperatorNamespace,
@@ -408,13 +401,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"KubeControllerManagerClient",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -450,13 +441,11 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
-	certRotator, err = certrotation.NewCertRotationController(
+	certRotator = certrotation.NewCertRotationController(
 		"KubeSchedulerClient",
 		certrotation.SigningRotation{
 			Namespace:              operatorclient.OperatorNamespace,
@@ -492,10 +481,8 @@ func newCertRotationController(
 			EventRecorder: eventRecorder,
 		},
 		operatorClient,
+		eventRecorder,
 	)
-	if err != nil {
-		return nil, err
-	}
 	ret.certRotators = append(ret.certRotators, certRotator)
 
 	return ret, nil
@@ -520,18 +507,15 @@ func (c *CertRotationController) WaitForReady(stopCh <-chan struct{}) {
 	if err := c.syncInternalLoadBalancerHostnames(); err != nil {
 		panic(err)
 	}
-
-	for _, certRotator := range c.certRotators {
-		certRotator.WaitForReady(stopCh)
-	}
 }
 
 // RunOnce will run the cert rotation logic, but will not try to update the static pod status.
 // This eliminates the need to pass an OperatorClient and avoids dubious writes and status.
 func (c *CertRotationController) RunOnce() error {
 	errlist := []error{}
+	runOnceCtx := context.WithValue(context.Background(), certrotation.RunOnceContextKey, true)
 	for _, certRotator := range c.certRotators {
-		if err := certRotator.RunOnce(); err != nil {
+		if err := certRotator.Sync(runOnceCtx, factory.NewSyncContext("CertRotationController", c.recorder)); err != nil {
 			errlist = append(errlist, err)
 		}
 	}
